﻿Imports System.IO
Imports System.Net
Imports System.Web

Public Class VideoDownloader

#Region "Declarations"
    Private request As HttpWebRequest
    Private response As HttpWebResponse
    Private source As String
    Private strike As Integer = 0
#End Region
#Region "Generate You-Tube Download URL"
    ''' <summary>  
    '''<para>This will return the download link of a video from youtube that you pass into this sub.</para>  
    ''' </summary>  
    ''' <param name="inputurl">The website address of the You-Tube page you are looking to download the video of.</param>
    ''' <param name="outputurl">A value that will eventually contain the created download link.</param>  
    ''' <param name="type">Returns the file type of the video. Either .mp4 or .flv depending on the video.</param>  
    ''' <param name="size">Returns the file size of the video that you will be downloading in bytes.</param> 
    Public Sub MakeDownloadURL(ByVal inputurl As String, ByRef outputurl As String, Optional ByRef type As String = "", Optional ByRef size As Integer = 0)
        'Perform cleanup to remove any 'leftovers' from previous runs.
        Cleanup()

        'Download the page that is entered.
        Try
            request = DirectCast(HttpWebRequest.Create(inputurl.ToString), HttpWebRequest)
        Catch
            type = ""
            Exit Sub
        End Try
        Try
            response = DirectCast(request.GetResponse, HttpWebResponse)
        Catch
            type = ""
            Exit Sub
        End Try
        'Grab the content of the sourcecode from the downloaded page.
        Try
            source = New StreamReader(response.GetResponseStream, System.Text.Encoding.Default).ReadToEnd
        Catch
            type = ""
            Exit Sub
        End Try

        'If the page contains "video_id" then we know it is a valid You-Tube Page.
        'Else we need to throw an error.
        If source.IndexOf("video_id") > -1 Then
            Dim signature As String = source
            'This grabs the .flv or .mp4 URL of the video, ready for doownloading.
            If signature.Contains("&fmt_url_map=") = True Then
                Try
                    signature = System.Text.RegularExpressions.Regex.Split(signature, "&fmt_url_map=")(1)
                Catch
                End Try
            ElseIf signature.Contains(Chr(34) + "fmt_url_map" + Chr(34) + ": " + Chr(34)) = True Then
                Try
                    signature = System.Text.RegularExpressions.Regex.Split(signature, Chr(34) + "fmt_url_map" + Chr(34) + ": " + Chr(34))(1)
                Catch
                End Try
            Else
                'A download URL could not be found.
                Debug.WriteLine("Critical Error: Download URL could not be produced.")
                type = ""
                Exit Sub
            End If

            'This changes the URL to conform to internet standards eg. %20 = " "
            ' %2F = "/"
            ' %2C = ","
            signature = HttpUtility.UrlDecode(signature)
            signature = signature.Replace("%2C", ",")

            signature = System.Text.RegularExpressions.Regex.Split(signature, "http://")(1)
            signature = signature.Insert(0, "http://")

            signature = signature.Replace("|", "")

            signature = signature.Remove((signature.LastIndexOf(",")), (signature.Length - signature.LastIndexOf(",")))

            'Checks for and Removes any remaining C characters from the string.
            If signature.Chars(signature.Length - 1) = "C" Then
                signature = signature.Remove(signature.Length - 1, 1)
            End If

            'Checks for videos such as http://www.youtube.com/watch?v=LvLCvdeWcsw that would not download due to a new layout of download data.
            If signature.Contains("rv.2.rating") Then
                Dim n As Integer = signature.IndexOf(ChrW(34) + ",")
                signature = signature.Remove(n, signature.Length - n)
            End If

            'PERFORMS CLEANUP
            Cleanup()
            Try
                request = DirectCast(HttpWebRequest.Create(signature), HttpWebRequest)
            Catch
                type = ""
                Exit Sub
            End Try
            response = DirectCast(request.GetResponse, HttpWebResponse)
            If response.ContentType = "video/x-flv" Then
                type = ".flv"
                outputurl = signature
                size = response.ContentLength
            ElseIf response.ContentType = "video/mp4" Then
                type = ".mp4"
                outputurl = signature
                size = response.ContentLength
            Else
                type = ""
                outputurl = ""
                Exit Sub
            End If
        End If
    End Sub
#End Region
#Region "Remove Invalid Characters from Video Title"
    ''' <summary>  
    '''<para>This can be used to remove invalid characters from a video title that might prevent it from being a valid filename.</para>  
    ''' </summary>  
    ''' <param name="videotitle">The video title that needs to be modified to have invalid characters removed from it.</param>
    Public Sub RemoveInvalidCharacters(ByRef videotitle As String)
        If videotitle.Contains("â™¥") Then
            videotitle = videotitle.Replace("â™¥", "")
        End If
        If videotitle.Contains("&amp") Then
            videotitle = videotitle.Replace("&amp", "")
        End If
        If videotitle.Contains(":") Then
            videotitle = videotitle.Replace(":", "")
        End If
        If videotitle.Contains(";") Then
            videotitle = videotitle.Replace(";", "")
        End If
        If videotitle.Contains("+") Then
            videotitle = videotitle.Replace("+", "")
        End If
        If videotitle.Contains("=") Then
            videotitle = videotitle.Replace("=", "")
        End If
        If videotitle.Contains("'") Then
            videotitle = videotitle.Replace("'", "")
        End If
        If videotitle.Contains("[") Then
            videotitle = videotitle.Replace("[", "")
        End If
        If videotitle.Contains("]") Then
            videotitle = videotitle.Replace("]", "")
        End If
        If videotitle.Contains("{") Then
            videotitle = videotitle.Replace("{", "")
        End If
        If videotitle.Contains("}") Then
            videotitle = videotitle.Replace("}", "")
        End If
        If videotitle.Contains("*") Then
            videotitle = videotitle.Replace("*", "")
        End If
        If videotitle.Contains("?") Then
            videotitle = videotitle.Replace("?", "")
        End If
        If videotitle.Contains("<") Then
            videotitle = videotitle.Replace("<", "")
        End If
        If videotitle.Contains(">") Then
            videotitle = videotitle.Replace(">", "")
        End If
        If videotitle.Contains("^") Then
            videotitle = videotitle.Replace("^", "")
        End If
        If videotitle.Contains("|") Then
            videotitle = videotitle.Replace("|", "")
        End If
        If videotitle.Contains("/") Then
            videotitle = videotitle.Replace("/", "")
        End If
        If videotitle.Contains("\") Then
            videotitle = videotitle.Replace("\", "")
        End If
        If videotitle.Contains("\\") Then
            videotitle = videotitle.Replace("\\", "")
        End If
        If videotitle.Contains(".") Then
            videotitle = videotitle.Replace(".", "")
        End If
        If videotitle.Contains(",") Then
            videotitle = videotitle.Replace(",", "")
        End If
        If videotitle.Contains(Chr(34)) Then
            videotitle = videotitle.Replace(Chr(34), "")
        End If
        If videotitle.Contains("@") Then
            videotitle = videotitle.Replace("@", "")
        End If
        If videotitle.Contains("¬") Then
            videotitle = videotitle.Replace("¬", "")
        End If
        If videotitle.Contains("£") Then
            videotitle = videotitle.Replace("£", "")
        End If
        If videotitle.Contains("$") Then
            videotitle = videotitle.Replace("$", "")
        End If
        If videotitle.Contains("%") Then
            videotitle = videotitle.Replace("%", "")
        End If
        If videotitle.Contains("^") Then
            videotitle = videotitle.Replace("^", "")
        End If
        If videotitle.Contains("!") Then
            videotitle = videotitle.Replace("!", "")
        End If
        If videotitle.Contains("?") Then
            videotitle = videotitle.Replace("?", "")
        End If
        If videotitle.Contains("#") Then
            videotitle = videotitle.Replace("#", "")
        End If
    End Sub
#End Region
#Region "CleanUp Code to Reset Requests."
    Public Sub Cleanup()
        'Code designed to clean up any previous data downloaded in previous uses.
        If request IsNot Nothing Then
            Try
                request.Abort()
            Catch
            End Try
        End If
        If response IsNot Nothing Then
            Try
                response.Close()
            Catch
            End Try
        End If
        Try
            source = ""
        Catch
        End Try
    End Sub
#End Region

    '////////////////////////////////////////////////////////
    'Obsolete Functions.
    '////////////////////////////////////////////////////////
#Region "Get YouTube Video's Title and a Video Thumbnail URL"
    ''' <summary>  
    '''<para>This will return the name of the video you are trying to download.</para>  
    ''' <Obsolete>Please now use the offical You-Tube API for this purpose! http://code.google.com/apis/youtube/2.0/developers_guide_dotnet.html </Obsolete> 
    ''' </summary>  
    ''' <param name="URL">The website address of the You-Tube page you are looking to get information from.</param>
    ''' <param name="title">A value that will eventually contain the You-Tube video title.</param>  
    Public Sub GetVideoTitle(ByVal URL As String, ByRef title As String)
        'Basically sometimes the title of the video can be retrieved by looking at the content near the video download link in a You-Tube Page.
        'It's not always 100% reliable and changes at You-Tube often cause this to fail.
        'The strike system tries to find a title, else downloads another copy of the page and looks again in a different way.
        title = "<html lang="
        'Reset strikes to 0.
        strike = 0
        Do While title.Contains("<html lang=")
            strike = strike + 1
            URL = URL.Replace("http://www.youtube.com/v/", "http://www.youtube.com/watch?v=")
            'PERFORMS CLEANUP
            Cleanup()
            'Download the HTML Source of the You-Tube Page.
            Try
                request = DirectCast(HttpWebRequest.Create(URL.ToString), HttpWebRequest)
            Catch
                'Somethings failed. Return Error.
                title = "Error"
                Exit Sub
            End Try

            Try
                response = DirectCast(request.GetResponse, HttpWebResponse)
            Catch
                Exit Sub
            End Try

            'Read the HTML source of the You-Tube Page
            Try
                source = New StreamReader(response.GetResponseStream, System.Text.Encoding.Default).ReadToEnd
            Catch
                'Somethings failed. Return Error.
                title = "Error"
                Exit Sub
            End Try

            Dim sourcetitle As String = source
            Try
                'Looks for the title of the You-Tube Video.
                If sourcetitle.Contains("meta name=" + Chr(34) + "title" + Chr(34)) Then
                    sourcetitle = System.Text.RegularExpressions.Regex.Split(sourcetitle, "<meta name=" + Chr(34) + "title" + Chr(34))(1)
                    Dim i2 As Integer
                    i2 = sourcetitle.IndexOf(">")
                    sourcetitle = sourcetitle.Remove(i2, sourcetitle.Length - i2)
                    sourcetitle = sourcetitle.Replace("&quot;", Chr(34))
                    sourcetitle = sourcetitle.Replace("content=" + Chr(34), "")
                    sourcetitle = sourcetitle.Replace(Chr(34), "")
                    title = sourcetitle
                ElseIf sourcetitle.Contains("<title>") Then
                    sourcetitle = System.Text.RegularExpressions.Regex.Split(sourcetitle, "<title>")(1)
                    sourcetitle = System.Text.RegularExpressions.Regex.Split(sourcetitle, "</title>")(0)
                    sourcetitle = sourcetitle.Replace(Chr(9), "")
                    sourcetitle = sourcetitle.Replace(Chr(13), "")
                    sourcetitle = sourcetitle.Replace(Chr(10), "")
                    sourcetitle = sourcetitle.Replace("YouTube- ", "")
                    title = sourcetitle
                    If sourcetitle.Contains("Broadcast Yourself") Then
                        'Somethings failed. Return Error.
                        title = "Error"
                        Exit Sub
                    End If
                Else
                    'Somethings failed. Return Error.
                    title = "Error"
                    Exit Sub
                End If
            Catch
                'Somethings failed. Return Error.
                title = "Error"
                Exit Sub
            End Try

            If strike = 2 Then
                'Somethings failed. Return Error.
                title = "Error"
                Exit Do
            End If
        Loop

        Do Until title.Chars(0) <> " "
            'Remove any spaces at the front of the returned video title.
            title = title.Remove(0, 1)
        Loop

    End Sub

#End Region
#Region "Get URL for You-Tube Video Thumbnail"
    ''' <summary>  
    '''<para>This will return a URL for a thumbnail of a video you are trying to download.</para>  
    ''' </summary> 
    ''' <Obsolete>Please now use the offical You-Tube API for this purpose! http://code.google.com/apis/youtube/2.0/developers_guide_dotnet.html </Obsolete> 
    ''' <param name="URL">The website address of the You-Tube page you are looking to get information from.</param>
    ''' <param name="PictureURL">A value that will eventually contain the video thumbnail URL.</param>  
    Public Sub GetPreviewThumbnail(ByVal URL As String, ByRef PictureURL As String)
        'Creates the preview video thumbnail URL for the particular video.
        PictureURL = "http://i1.ytimg.com/vi/" + URL.Replace("http://www.youtube.com/watch?v=", "").ToString + "/default.jpg"
    End Sub
#End Region
    '////////////////////////////////////////////////////////

End Class
